@@ -21,9 +21,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDir>
 #include <QFile>
 #include <QJsonDocument>
+#include <QStringBuilder>
 
 #include <kscreen/config.h>
-#include <kscreen/output.h>
 
 QString Control::s_dirName = QStringLiteral("control/");
 
@@ -539,6 +539,143 @@ void ControlConfig::setReplicationSource(const QString &outputId, const QString
     // TODO: shall we set this information also as new global value (like with auto-rotate)?
 }
 
+uint32_t ControlConfig::getOverscan(const KScreen::OutputPtr &output) const
+{
+    return getOverscan(output->hashMd5(), output->name());
+}
+
+uint32_t ControlConfig::getOverscan(const QString &outputId, const QString &outputName) const
+{
+    const auto retention = getOutputRetention(outputId, outputName);
+    if (retention == OutputRetention::Individual) {
+        const QVariantList outputsInfo = getOutputs();
+        for (const auto &variantInfo : outputsInfo) {
+            const QVariantMap info = variantInfo.toMap();
+            if (!infoIsOutput(info, outputId, outputName)) {
+                continue;
+            }
+            const auto val = info[QStringLiteral("overscan")];
+            if (val.canConvert<uint>()) {
+                return static_cast<uint32_t>(val.toUInt());
+            } else {
+                return 0;
+            }
+        }
+    }
+    // Retention is global or info for output not in config control file.
+    if (auto *outputControl = getOutputControl(outputId, outputName)) {
+        return outputControl->overscan();
+    }
+
+    // Info for output not found.
+    return 0;
+}
+
+void ControlConfig::setOverscan(const KScreen::OutputPtr &output, const uint32_t value)
+{
+    setOverscan(output->hashMd5(), output->name(), value);
+}
+
+void ControlConfig::setOverscan(const QString &outputId, const QString &outputName, const uint32_t value)
+{
+    QList<QVariant>::iterator it;
+    QVariantList outputsInfo = getOutputs();
+
+    auto setOutputOverscan = [&outputId, &outputName, value, this]() {
+        if (auto *control = getOutputControl(outputId, outputName)) {
+            control->setOverscan(value);
+        }
+    };
+
+    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
+        QVariantMap outputInfo = (*it).toMap();
+        if (!infoIsOutput(outputInfo, outputId, outputName)) {
+            continue;
+        }
+        outputInfo[QStringLiteral("overscan")] = value;
+        *it = outputInfo;
+        setOutputs(outputsInfo);
+        setOutputOverscan();
+        return;
+    }
+    // no entry yet, create one
+    auto outputInfo = createOutputInfo(outputId, outputName);
+    outputInfo[QStringLiteral("overscan")] = value;
+
+    outputsInfo << outputInfo;
+    setOutputs(outputsInfo);
+    setOutputOverscan();
+}
+
+KScreen::Output::VrrPolicy ControlConfig::getVrrPolicy(const KScreen::OutputPtr &output) const
+{
+    return getVrrPolicy(output->hashMd5(), output->name());
+}
+
+KScreen::Output::VrrPolicy ControlConfig::getVrrPolicy(const QString &outputId, const QString &outputName) const
+{
+    const auto retention = getOutputRetention(outputId, outputName);
+    if (retention == OutputRetention::Individual) {
+        const QVariantList outputsInfo = getOutputs();
+        for (const auto &variantInfo : outputsInfo) {
+            const QVariantMap info = variantInfo.toMap();
+            if (!infoIsOutput(info, outputId, outputName)) {
+                continue;
+            }
+            const auto val = info[QStringLiteral("vrrpolicy")];
+            if (val.canConvert<uint>()) {
+                return static_cast<KScreen::Output::VrrPolicy>(val.toUInt());
+            } else {
+                return KScreen::Output::VrrPolicy::Automatic;
+            }
+        }
+    }
+    // Retention is global or info for output not in config control file.
+    if (auto *outputControl = getOutputControl(outputId, outputName)) {
+        return outputControl->vrrPolicy();
+    }
+
+    // Info for output not found.
+    return KScreen::Output::VrrPolicy::Automatic;
+}
+
+void ControlConfig::setVrrPolicy(const KScreen::OutputPtr &output, const KScreen::Output::VrrPolicy value)
+{
+    setVrrPolicy(output->hashMd5(), output->name(), value);
+}
+
+void ControlConfig::setVrrPolicy(const QString &outputId, const QString &outputName, const KScreen::Output::VrrPolicy enumValue)
+{
+    QList<QVariant>::iterator it;
+    QVariantList outputsInfo = getOutputs();
+    uint value = static_cast<uint>(enumValue);
+
+    auto setOutputVrr = [&outputId, &outputName, enumValue, this]() {
+        if (auto *control = getOutputControl(outputId, outputName)) {
+            control->setVrrPolicy(enumValue);
+        }
+    };
+
+    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
+        QVariantMap outputInfo = (*it).toMap();
+        if (!infoIsOutput(outputInfo, outputId, outputName)) {
+            continue;
+        }
+        outputInfo[QStringLiteral("vrrpolicy")] = value;
+        *it = outputInfo;
+        setOutputs(outputsInfo);
+        setOutputVrr();
+        return;
+    }
+    // no entry yet, create one
+    auto outputInfo = createOutputInfo(outputId, outputName);
+    outputInfo[QStringLiteral("vrrpolicy")] = value;
+
+    outputsInfo << outputInfo;
+    setOutputs(outputsInfo);
+    setOutputVrr();
+}
+
 QVariantList ControlConfig::getOutputs() const
 {
     return constInfo()[QStringLiteral("outputs")].toList();
@@ -633,3 +770,39 @@ void ControlOutput::setAutoRotateOnlyInTabletMode(bool value)
     }
     infoMap[QStringLiteral("autorotate-tablet-only")] = value;
 }
+
+uint32_t ControlOutput::overscan() const
+{
+    const auto val = constInfo()[QStringLiteral("overscan")];
+    if (val.canConvert<uint>()) {
+        return val.toUInt();
+    }
+    return 0;
+}
+
+void ControlOutput::setOverscan(uint32_t value)
+{
+    auto &infoMap = info();
+    if (infoMap.isEmpty()) {
+        infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
+    }
+    infoMap[QStringLiteral("overscan")] = static_cast<uint>(value);
+}
+
+KScreen::Output::VrrPolicy ControlOutput::vrrPolicy() const
+{
+    const auto val = constInfo()[QStringLiteral("vrrpolicy")];
+    if (val.canConvert<uint>()) {
+        return static_cast<KScreen::Output::VrrPolicy>(val.toUInt());
+    }
+    return KScreen::Output::VrrPolicy::Automatic;
+}
+
+void ControlOutput::setVrrPolicy(KScreen::Output::VrrPolicy value)
+{
+    auto &infoMap = info();
+    if (infoMap.isEmpty()) {
+        infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
+    }
+    infoMap[QStringLiteral("vrrpolicy")] = static_cast<uint>(value);
+}
